import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Form, Row, Col, Input, Select, Radio, notification, Tooltip, InputNumber, Cascader } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'

import { formRule } from '@/utils/option.js'
import asyncComponent from '@/utils/asyncComponent'
import Utils from '@/utils/utils.js'
import './index.scss'

const MkEditIcon = asyncComponent(() => import('@/components/mkIcon'))

const { TextArea } = Input
const actionTypeOptions = {
  pop: ['label', 'position', 'OpenType', 'intertype', 'Ot', 'icon', 'class', 'execSuccess', 'execError', 'openmenu', 'output', 'tipTitle'],
  prompt: ['label', 'position', 'OpenType', 'intertype', 'Ot', 'icon', 'class', 'execSuccess', 'execError', 'openmenu', 'output', 'tipTitle'],
  exec: ['label', 'position', 'OpenType', 'intertype', 'Ot', 'icon', 'class', 'execSuccess', 'execError', 'openmenu', 'output'],
  excelIn: ['label', 'Ot', 'OpenType', 'intertype', 'icon', 'class', 'sheet', 'execSuccess', 'execError'],
  excelOut: ['label', 'OpenType', 'intertype', 'icon', 'class', 'execSuccess', 'execError', 'pagination', 'search'],
  popview: ['label', 'Ot', 'OpenType', 'icon', 'class', 'position', 'tabType', 'linkTab', 'popClose', 'display', 'ratio', 'clickouter'],
  tab: ['label', 'Ot', 'OpenType', 'icon', 'class', 'position', 'linkmenu'],
  innerpage: ['label', 'Ot', 'OpenType', 'pageTemplate', 'icon', 'class', 'position'],
  funcbutton: ['label', 'OpenType', 'funcType', 'icon', 'class']
}

class MainSearch extends Component {
  static propTpyes = {
    dict: PropTypes.object,      // 字典项
    setting: PropTypes.object,   // 页面设置
    formlist: PropTypes.any,     // 表单信息
    card: PropTypes.any,         // 按钮信息
    inputSubmit: PropTypes.any   // 回车提交事件
  }

  state = {
    formlist: null,  // 表单信息
    requireOptions: [{
      value: 'notRequired',
      text: this.props.dict['header.form.notRequired']
    }, {
      value: 'requiredSgl',
      text: this.props.dict['header.form.requiredSgl']
    }, {
      value: 'required',
      text: this.props.dict['header.form.required']
    }, {
      value: 'requiredOnce',
      text: this.props.dict['header.form.requiredOnce']
    }],
    insertUpdateOptions: [{
      value: 'insert',
      text: this.props.dict['header.form.action.insert']
    }, {
      value: 'update',
      text: this.props.dict['header.form.action.update']
    }, {
      value: 'audit',
      text: this.props.dict['header.form.action.audit']
    }],
    deleteOptions: [{
      value: 'LogicDelete',
      text: '逻辑删除'
    }, {
      value: 'delete',
      text: '物理删除'
    }, {
      value: 'custom',
      text: this.props.dict['header.form.custom']
    }],
    interTypeOptions: [{
      value: 'system',
      text: this.props.dict['model.interface.system']
    }, {
      value: 'inner',
      text: this.props.dict['model.interface.inner']
    }, {
      value: 'outer',
      text: this.props.dict['model.interface.outer']
    }, {
      value: 'custom',
      text: '自定义'
    }]
  }

  record = {}
  
  UNSAFE_componentWillMount () {
    this.props.formlist.forEach(item => {
      this.record[item.key] = item.initVal
    })

    let { shows, reOptions, reTypes, reRequired, reReadonly } = this.getMutilOptions()

    this.setState({
      formlist: this.props.formlist.map(item => {
        item.hidden = !shows.includes(item.key)
        item.initVal = this.record[item.key]

        if (reOptions[item.key]) {
          item.options = reOptions[item.key]
        }
        if (reTypes[item.key]) {
          item.type = reTypes[item.key]
        }
        if (reRequired[item.key] !== undefined) {
          item.required = reRequired[item.key]
        }
        if (reReadonly[item.key] !== undefined) {
          item.readonly = reReadonly[item.key]
        }

        return item
      })
    })
  }

  componentDidMount () {
    const { card } = this.props

    if (card.focus) {
      try {
        let _form = document.getElementById('label')
        _form.select()
      } catch (e) {
        console.warn('表单focus失败！')
      }
    }
  }

  getMutilOptions = () => {
    const { requireOptions } = this.state

    let openType = this.record.OpenType
    let Ot = this.record.Ot
    let shows = fromJS(actionTypeOptions[openType] || []).toJS()
    let reOptions = {}
    let reTypes = {}
    let reRequired = {}
    let reReadonly = {}

    if (openType === 'pop' || openType === 'prompt' || openType === 'exec') {
      let intertype = this.record.intertype

      reOptions.intertype = this.state.interTypeOptions

      if (intertype === 'custom') {
        shows.push('procMode', 'interface', 'callbackType', 'cbTable', 'proInterface', 'method', 'cross')
        if (this.record.procMode === 'system') {
          shows.push('sql', 'sqlType')
        } else {
          reRequired.innerFunc = true
          shows.push('innerFunc')
        }
        reReadonly.interface = false
        reRequired.interface = true
      } else if (intertype === 'outer') {
        shows.push('innerFunc', 'sysInterface', 'interface', 'proInterface', 'outerFunc', 'callbackFunc', 'output')
        reRequired.innerFunc = false

        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true
        } else {
          reReadonly.interface = true
          reRequired.interface = false
        }
      } else if (intertype === 'inner') {
        shows.push('innerFunc', 'output')
        reRequired.innerFunc = true
      } else {
        shows.push('sql', 'sqlType', 'output')
      }

      if (this.record.sqlType === 'insert') {
        reOptions.Ot = requireOptions.filter(op => op.value === 'notRequired')
      } else {
        reOptions.Ot = requireOptions
      }

      if (openType === 'prompt' || openType === 'exec') {
        reOptions.sqlType = this.state.deleteOptions
      } else {
        reOptions.sqlType = this.state.insertUpdateOptions
      }
    } else if (openType === 'excelIn') {
      reOptions.intertype = this.state.interTypeOptions.filter(op => op.value !== 'custom')
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      if (this.record.intertype === 'outer') {
        shows.push('innerFunc', 'sysInterface', 'interface', 'proInterface', 'outerFunc', 'callbackFunc')
        reRequired.innerFunc = false

        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true
        } else {
          reReadonly.interface = true
          reRequired.interface = false
        }
      } else if (this.record.intertype === 'inner') {
        shows.push('innerFunc')
        reRequired.innerFunc = true
      }
    } else if (openType === 'excelOut') {
      reOptions.intertype = this.state.interTypeOptions.filter(op => op.value !== 'custom')

      if (this.record.intertype === 'outer') {
        shows.push('innerFunc', 'sysInterface', 'interface', 'proInterface', 'outerFunc')
        reRequired.innerFunc = false

        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true
        } else {
          reReadonly.interface = true
          reRequired.interface = false
        }
      } else if (this.record.intertype === 'inner') {
        shows.push('innerFunc')
        reRequired.innerFunc = true
      }
    } else if (openType === 'popview') {
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      if (this.record.display === 'drawer') {
        shows.push('placement')
      }
    } else if (openType === 'tab') {
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      reRequired.linkmenu = true
    } else if (openType === 'innerpage') {
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      if (this.record.pageTemplate === 'custom') {
        shows.push('url', 'joint')
      } else if (this.record.pageTemplate === 'billprint') {
        shows.push('printTemp')
        reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl', 'required'].includes(op.value))
      } else if (this.record.pageTemplate === 'pay' || this.record.pageTemplate === 'print' || this.record.pageTemplate === 'billprintTemp') {
        reOptions.Ot = requireOptions.filter(op => op.value === 'requiredSgl')
      }
    } else if (openType === 'funcbutton') {
      reOptions.intertype = this.state.interTypeOptions.filter(op => op.value !== 'custom')
      reOptions.Ot = requireOptions

      let _funcType = this.record.funcType

      if (_funcType === 'print') {
        shows.push('execMode', 'intertype', 'Ot', 'execSuccess', 'execError', 'resetPageIndex')
        if (this.record.intertype === 'outer') {
          shows.push('innerFunc', 'sysInterface', 'interface', 'proInterface', 'outerFunc', 'callbackFunc')
          reRequired.innerFunc = false

          if (this.record.sysInterface === 'false') {
            reReadonly.interface = false
            reRequired.interface = true
          } else {
            reReadonly.interface = true
            reRequired.interface = false
          }
        } else if (this.record.intertype === 'inner') {
          shows.push('innerFunc')
          reRequired.innerFunc = true
        }
      } else if (_funcType === 'closetab') {
        shows.push('refreshTab')
      }
    }

    if (this.record.position === 'grid') { // 表格中按钮只能选单行
      reOptions.Ot = requireOptions.filter(op => op.value === 'requiredSgl')
    }
    
    if (Ot !== 'notRequired') {
      reOptions.control = [
        { value: '', text: '无' },
        { value: 'disabled', text: '禁用' },
        { value: 'hidden', text: '隐藏' },
        { value: 'parent', text: '上级' }
      ]
    } else {
      reOptions.control = [
        { value: '', text: '无' },
        { value: 'parent', text: '上级' }
      ]
    }
    shows.push('control')
    if (this.record.control === 'parent') {
      reTypes.controlField = 'text'
    } else {
      reTypes.controlField = 'select'
    }
    if (this.record.control) {
      shows.push('controlField', 'controlVal')
    }
    if (this.record.control === 'disabled') {
      shows.push('reason')
    }

    return {
      shows,
      reOptions,
      reTypes,
      reRequired,
      reReadonly
    }
  }

  /**
   * @description 切换
   */
  optionChange = (key, value) => {
    this.record[key] = value
    let _fieldval = {}

    if (key === 'OpenType') {
      this.record.intertype = 'system'
      this.record.procMode = 'system'
      this.record.display = 'modal'
      this.record.position = 'toolbar'
      this.record.pageTemplate = ''
      this.record.funcType = ''
      this.record.sqlType = ''

      if (value === 'pop' || value === 'prompt' || value === 'exec') {
        _fieldval.intertype = 'system'
        _fieldval.sqlType = ''
  
      } else if (value === 'excelIn') {
        _fieldval.intertype = 'system'
        _fieldval.Ot = 'notRequired'
        
        _fieldval.label = this.props.dict['model.form.excelIn']
        _fieldval.class = 'dgreen'
        this.record.Ot = 'notRequired'
        this.record.label = this.props.dict['model.form.excelIn']
        this.record.class = 'dgreen'
       
  
      } else if (value === 'excelOut') {
        _fieldval.intertype = 'system'
        _fieldval.label = this.props.dict['model.form.excelOut']
        _fieldval.class = 'dgreen'
        this.record.Ot = 'notRequired'
        this.record.label = this.props.dict['model.form.excelOut']
        this.record.class = 'dgreen'
  
      } else if (value === 'popview') {
        _fieldval.display = 'modal'
        _fieldval.Ot = 'requiredSgl'
        _fieldval.popClose = 'grid'
        this.record.display = 'modal'
        this.record.Ot = 'requiredSgl'
        this.record.popClose = 'grid'
  
      } else if (value === 'tab') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
  
      } else if (value === 'innerpage') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
        
      }
      
      if (value === 'excelIn') {
        _fieldval.label = this.props.dict['model.form.excelIn']
        _fieldval.class = 'border-dgreen'
      } else if (value === 'excelOut') {
        _fieldval.label = this.props.dict['model.form.excelOut']
        _fieldval.class = 'dgreen'
        _fieldval.control = ''
        this.record.control = ''
      }

      if (_fieldval.Ot === 'notRequired') {
        this.record.control = ''
        _fieldval.control = ''
      }

      if (!['funcbutton', 'excelIn', 'excelOut'].includes(value)) {
        _fieldval.position = 'toolbar'
      }
    } else if (key === 'position') {
      if (value === 'grid') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
      } else if (['pop', 'prompt', 'exec'].includes(this.record.OpenType) && this.record.sqlType === 'insert') {
        _fieldval.Ot = 'notRequired'
        this.record.Ot = 'notRequired'
      }
    } else if (key === 'funcType') {
      if (value === 'print') {
        _fieldval.label = '打印'
      } else if (value === 'closetab') {
        _fieldval.label = '关闭'
      }
    } else if (key === 'sqlType') {
      if (value === 'insert') {
        _fieldval.label = '添加'
        _fieldval.Ot = 'notRequired'
        _fieldval.class = 'green'
      } else if (value === 'update') {
        _fieldval.label = '修改'
        _fieldval.Ot = 'requiredSgl'
        _fieldval.class = 'purple'
      } else if (value === 'audit') {
        _fieldval.label = '审核'
        _fieldval.Ot = 'requiredSgl'
        _fieldval.class = 'primary'
      } else if (value === 'LogicDelete' || value === 'delete') {
        _fieldval.label = '删除'
        _fieldval.Ot = 'requiredSgl'
        _fieldval.class = 'danger'
      }

      if (this.record.position === 'grid') {
        _fieldval.Ot = 'requiredSgl'
      }

      this.record.label = _fieldval.label
      this.record.Ot = _fieldval.Ot
      this.record.class = _fieldval.class || ''

      if (_fieldval.Ot === 'notRequired') {
        this.record.control = ''
        _fieldval.control = ''
      }
      
    } else if (key === 'pageTemplate') {
      if (value === 'pay') {
        _fieldval.Ot = 'requiredSgl'
      } else if (value === 'billprint') {
        _fieldval.Ot = 'requiredSgl'
      }
    } else if (key === 'intertype') {
      this.record.sysInterface = 'false'
    } else if (key === 'sysInterface') {
      if (value === 'true') {
        _fieldval.interface = window.GLOB.mainSystemApi || ''
        this.record.interface = window.GLOB.mainSystemApi || ''
      }
    }

    let { shows, reOptions, reTypes, reRequired, reReadonly } = this.getMutilOptions()

    this.setState({
      formlist: this.state.formlist.map(item => {
        item.hidden = !shows.includes(item.key)
        item.initVal = this.record[item.key]

        if (reOptions[item.key]) {
          item.options = reOptions[item.key]
        }
        if (reTypes[item.key]) {
          item.type = reTypes[item.key]
        }
        if (reRequired[item.key] !== undefined) {
          item.required = reRequired[item.key]
        }
        if (reReadonly[item.key] !== undefined) {
          item.readonly = reReadonly[item.key]
        }

        return item
      })
    }, () => {
      this.props.form.setFieldsValue(_fieldval)
    })
  }

  handleSubmit = (e) => {
    e.preventDefault()

    if (this.props.inputSubmit) {
      this.props.inputSubmit()
    }
  }

  getFields() {
    const { dict } = this.props
    const { getFieldDecorator } = this.props.form
    const fields = []

    this.state.formlist.forEach((item, index) => {
      if (item.hidden) return

      let span = 12
      let rules = []
      let className = ''
      let content = null
      let initVal = item.initVal || ''

      if (item.type === 'text') {
        rules = [
          { required: item.required, message: dict['form.required.input'] + item.label + '!' }
        ]
  
        if (item.key === 'innerFunc') {
          let str = '^(' + item.fields.join('|') + ')'
          let _patten = new RegExp(str + formRule.func.innerPattern + '$', 'g')
          rules.push(
            { pattern: _patten, message: formRule.func.innerMessage },
            { max: formRule.func.max, message: formRule.func.maxMessage }
          )
        } else if (item.key === 'outerFunc' || item.key === 'callbackFunc') {
          rules.push(
            { pattern: formRule.func.pattern, message: formRule.func.message },
            { max: formRule.func.max, message: formRule.func.maxMessage }
          )
        } else {
          rules.push({ max: formRule.input.max, message: formRule.input.message })
        }

        content = <Input placeholder="" autoComplete="off" onPressEnter={this.handleSubmit} />
      } else if (item.type === 'number') {
        rules = [
          { required: item.required, message: dict['form.required.input'] + item.label + '!' }
        ]

        content = <InputNumber min={item.min} max={item.max} precision={item.precision} onPressEnter={this.handleSubmit}/>
      } else if (item.type === 'select') {
        rules = [
          { required: item.required, message: dict['form.required.select'] + item.label + '!' }
        ]

        content = <Select
          showSearch
          allowClear={item.allowClear === true}
          filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
          onChange={(value) => {this.optionChange(item.key, value)}}
          getPopupContainer={() => document.getElementById('winter')}
        >
          {item.options.map((option, index) =>
            <Select.Option key={index} value={(option.value || option.field || '')}>
              {(option.text || option.label)}
            </Select.Option>
          )}
        </Select>
      } else if (item.type === 'radio') {
        rules = [
          { required: item.required, message: dict['form.required.select'] + item.label + '!' }
        ]

        content = <Radio.Group onChange={(e) => {this.optionChange(item.key, e.target.value)}}>
          {item.options.map(option => <Radio key={option.value} value={option.value}>{option.text}</Radio>)}
        </Radio.Group>
      } else if (item.type === 'cascader') {
        initVal = item.initVal || []
        rules = [
          { required: item.required, message: dict['form.required.select'] + item.label + '!' }
        ]

        content = <Cascader options={item.options || []} expandTrigger="hover" placeholder=""/>
      } else if (item.type === 'icon') {
        rules = [
          { required: item.required, message: dict['form.required.select'] + item.label + '!' }
        ]

        content = <MkEditIcon options={['edit', 'hint', 'direction', 'normal', 'data']} allowClear/>
      } else if (item.type === 'textarea') {
        span = 24
        className = 'textarea'
        rules = [
          { required: item.readonly ? false : item.required, message: dict['form.required.input'] + item.label + '!' }
        ]

        content = <TextArea rows={2} readOnly={item.readonly}/>
      }

      fields.push(
        <Col span={span} key={index}>
          <Form.Item className={className} label={item.tooltip ?
            <Tooltip placement="topLeft" title={item.tooltip}>
              <QuestionCircleOutlined className="mk-form-tip" />
              {item.label}
            </Tooltip> : item.label
          }>
            {getFieldDecorator(item.key, {
              initialValue: initVal,
              rules: rules
            })(content)}
          </Form.Item>
        </Col>
      )
    })
    return fields
  }

  handleConfirm = () => {
    const { setting, card } = this.props
    // 表单提交时检查输入值是否正确
    return new Promise((resolve, reject) => {
      this.props.form.validateFieldsAndScroll((err, values) => {
        if (!err) {
          values.uuid = card.uuid
          values.verify = card.verify || null
          values.position = values.position || 'toolbar'

          if (values.OpenType === 'excelOut') {
            if (values.intertype === 'system' && setting.interType !== 'system') { // 导出excel需使用查询数据源
              notification.warning({
                top: 92,
                message: '表格数据查询未使用系统函数，导出Excel不可使用系统函数！',
                duration: 5
              })
              return
            }
            
            values.Ot = 'notRequired'
          } else if (values.OpenType === 'popview' && !values.linkTab) { // 没有关联标签（新建时），创建新标签Id
            values.linkTab = Utils.getuuid()
          } else if (['pop', 'prompt', 'exec'].includes(values.OpenType) && values.verify) {
            if ((values.Ot === 'requiredOnce' || card.Ot === 'requiredOnce') && card.Ot !== values.Ot) {
              values.verify.uniques = []
            }
            if (card.Ot !== values.Ot) {
              if (values.Ot === 'notRequired') {
                values.verify.invalid = 'false'
              } else if (card.Ot === 'notRequired' && values.Ot !== 'notRequired') {
                values.verify.invalid = 'true'
              }
            }
          } else if (values.OpenType === 'tab' && values.linkmenu) {
            if (sessionStorage.getItem('thdMenuList')) {
              let list = null
              try {
                list = JSON.parse(sessionStorage.getItem('thdMenuList')) || []
              } catch (e) {
                list = []
              }

              let id = values.linkmenu[values.linkmenu.length - 1]
        
              list.forEach(item => {
                if (item.MenuID === id) {
                  values.MenuID = id
                  values.MenuName = item.MenuName
                  values.MenuNo = item.MenuNo
                  values.tabType = item.type
                }
              })
            }
          }
          if (values.openmenu && Array.isArray(values.openmenu) && values.openmenu.length > 0) {
            let list = null
            try {
              list = JSON.parse(sessionStorage.getItem('thdMenuList')) || []
            } catch (e) {
              list = []
            }

            let id = values.openmenu[values.openmenu.length - 1]

            list.forEach(item => {
              if (item.MenuID === id) {
                values.MenuID = id
                values.MenuName = item.MenuName
                values.MenuNo = item.MenuNo
                values.tabType = item.type
              }
            })
          }

          resolve(values)
        } else {
          reject(err)
        }
      })
    })
  }

  render() {
    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 }
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 17 }
      }
    }
    return (
      <Form {...formItemLayout} className="normal-action-list-form" id="winter">
        <Row gutter={24}>{this.getFields()}</Row>
      </Form>
    )
  }
}

export default Form.create()(MainSearch)